# -*- coding: utf-8 -*-
globalization = {
    "en": {
        "rerun failed scenario": u"rerun failed scenario",
        "information association of failed operation":
            "information association of failed operation, run the {} time"
            " :[{}]",
        "start record": "start record",
        "stop record": "stop record",
        "rank": "rank",
        "parent": "parent",
        "children": "children",
        "sibling": "sibling",
        "offsprings": "offsprings",
        "and": "and",
        "up": "up",
        "down": "down",
        "left": "left",
        "right": "right",
        "modal_list": ["name=同意并继续",
                       "name=使用App时允许",
                       "name=允许",
                       "name=common update X",
                       "name=我知道了",
                       "text=同意并继续",
                       "text=使用App时允许",
                       "text=允许",
                       "text=common update X",
                       "text=我知道了",
                       "text=System UI isn't responding",
                       "text=Wait"]
    },
    "zh-CN": {
        "rerun failed scenario": u"\u5931\u8d25\u91cd\u65b0\u8fd0\u884c",
        "information association of failed operation":
            "失败运行的信息关联,运行第{}次:[{}]",
        "start record": u"开始录屏",
        "stop record": u"结束录屏",
        "rank": u"第",
        "parent": u"父节点",
        "children": u"孩子",
        "sibling": u"兄弟",
        "offsprings": u"后代",
        "and": u"并且",
        "up": u"上",
        "down": u"下",
        "left": u"左",
        "right": u"右",
        "modal_list": ["name=同意并继续",
                       "name=使用App时允许",
                       "name=允许",
                       "name=common update X",
                       "name=我知道了",
                       "text=同意并继续",
                       "text=使用App时允许",
                       "text=允许",
                       "text=common update X",
                       "text=我知道了",
                       "text=System UI isn't responding",
                       "text=Wait"]
    },
}

step_language = {
    "en": {
        "install app[{param}]": ["install app[{param}]"],
        "start app[{param}]": ["start app[{param}]"],
    },
    "zh-CN": {
        "install app[{param}]": ["安装APP[{param}]"],
        "delete app[{param}]": ["删除APP[{param}]"],
        "start app[{param}]": ["启动APP[{param}]"],
        "restart app": ["重启App"],
        "close app": ["关闭App"],
        "init device[{param}]": ["设备初始化[{param}]"],
        "connect device[{param}]": ["连接设备[{param}]"],
        "start recording timeout[{param}]": ["开始录屏超时[{param}]"],
        "start record": ["开始录屏"],
        "stop record": ["结束录屏"],
        "go to url[{param}]": ["跳转到[{param}]", "跳转页面到[{param}]"],
        "return to previous page": ["返回上一页"],
        "go to home page": ["回到首页"],
        "logon account[{param1}]password[{param2}]": [
            "登录账号[{param1}]密码[{param2}]",
            "登陆账号[{param1}]密码[{param2}]",
        ],
        "logout": ["退出登陆", "退出登录"],
        "wait[{param}]seconds": ["等待[{param}]秒"],
        "screenshot": ["全屏截图"],
        "information association of failed operation, run the {param1} time"
        " :[{param2}]": ["失败运行的信息关联,运行第{param1}次:[{param2}]"],
        "text[{param1}]property[{param2}]is {param3}": [
            "文案[{param1}]的属性[{param2}]为{param3}"
        ],
        "element[{param1}]property[{param2}]is {param3}": [
            "元素[{param1}]的属性[{param2}]为{param3}"
        ],
        "click[{param}]": ["点击[{param}]"],
        "click text[{param}]": ["点击文案[{param}]"],
        "click position[{x},{y}]": ["点击屏幕位置[{x},{y}]"],
        "in[{param1}]input[{param2}]": ["在[{param1}]中输入[{param2}]"],
        "element[{param1}]position not change in[{param2}]seconds": [
            "元素[{param1}]位置[{param2}]秒内未变动"
        ],
        "[{param1}]slide to {param2} distance[{param3}]": [
            "[{param1}]向{param2}滑动[{param3}]"],
        "slide to {param1} distance[{param2}]": ["全屏向{param1}滑动[{param2}]"],
        "exist text[{param}]": ["存在[{param}]的文案"],
        "not exist text[{param}]": ["不存在[{param}]的文案"],
        "text[{param}]disappear": ["文案[{param}]消失"],
        "exist[{param}]element": ["存在[{param}]的元素"],
        "not exist element[{param}]": ["不存在[{param}]的元素"],
        "element[{param}]disappear": ["元素[{param}]消失"],
        "the text of element[{param1}]is[{param2}]": [
            "[{param1}]的文案为[{param2}]"
        ],
        "the text of element[{param1}]include[{param2}]": [
            "[{param1}]的文案包含[{param2}]"
        ],
        "page rendering complete appears element[{param}]": [
            "页面渲染完成出现元素[{param}]"],
        "existing element[{param}]": ["存在元素[{param}]"],

        "in[{param1}]from {param2} find[{param3}]element": [
            "在[{param1}]中向{param2}查找[{param3}]的元素"
        ],
        "from {param1} find[{param2}]element": ["向{param1}查找[{param2}]的元素"],
    },
}
