# -*- coding: utf-8 -*-
import flybirds.core.dsl.step.step_loader
import flybirds.core.dsl.step.app
import flybirds.core.dsl.step.common
import flybirds.core.dsl.step.element
import flybirds.core.dsl.step.page
import flybirds.core.dsl.step.device

# import custom step statements
import pscript.dsl.step.custom_test

# here import extend step statements