# -*- coding: utf-8 -*-
"""
The expansion of each hook function during the execution of flybirds
"""


def before_all_extend(context):
    pass


def after_all_extend(context):
    pass


def before_feature_extend(context, feature):
    pass


def after_feature_extend(context, feature):
    pass


def before_scenario_extend(context, scenario):
    pass


def after_scenario_extend(context, scenario):
    pass


def before_step_extend(context, step):
    pass


def after_step_extend(context, step):
    pass


def before_tag_extend(context, tag):
    pass


def after_tag_extend(context, tag):
    pass
